import { Test, TestingModule } from '@nestjs/testing';
import { AggregateID, NotFoundException } from '@mobicoop/ddd-library';
import { SetConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/set-configuration.request.dto';
import { Domain } from '@modules/configuration/core/domain/configuration.types';
import { SetConfigurationService } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.service';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { SetConfigurationCommand } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.command';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';

const setConfigurationRequest: SetConfigurationRequestDto = {
  domain: Domain.AD,
  key: 'seatsProposed',
  value: '3',
};

const existingConfigurationEntity = new ConfigurationEntity({
  id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
  props: {
    domain: Domain.AD,
    key: 'seatsProposed',
    value: '2',
  },
  createdAt: new Date('2023-10-23T07:00:00Z'),
  updatedAt: new Date('2023-10-23T07:00:00Z'),
});

const mockConfigurationRepository = {
  findOne: jest
    .fn()
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => existingConfigurationEntity)
    .mockImplementationOnce(() => existingConfigurationEntity),
  insert: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    }),
  update: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Set Configuration Service', () => {
  let setConfigurationService: SetConfigurationService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        SetConfigurationService,
      ],
    }).compile();

    setConfigurationService = module.get<SetConfigurationService>(
      SetConfigurationService,
    );
  });

  it('should be defined', () => {
    expect(setConfigurationService).toBeDefined();
  });

  describe('execution', () => {
    const setConfigurationCommand = new SetConfigurationCommand(
      setConfigurationRequest,
    );
    it('should create a new configuration item', async () => {
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      const result: AggregateID = await setConfigurationService.execute(
        setConfigurationCommand,
      );
      expect(result).toBe('047a6ecf-23d4-4d3e-877c-3225d560a8da');
    });
    it('should throw an error if something bad happens on configuration item creation', async () => {
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        setConfigurationService.execute(setConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
    it('should update an existing configuration item', async () => {
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      const result: AggregateID = await setConfigurationService.execute(
        setConfigurationCommand,
      );
      expect(result).toBe('047a6ecf-23d4-4d3e-877c-3225d560a8da');
    });
    it('should throw an error if something bad happens on configuration item update', async () => {
      ConfigurationEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        setConfigurationService.execute(setConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
