import {
  DatabaseErrorException,
  NotFoundException,
} from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { Domain } from '@modules/configuration/core/domain/configuration.types';
import { DeleteConfigurationGrpcController } from '@modules/configuration/interface/grpc-controllers/delete-configuration.grpc.controller';
import { DeleteConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/delete-configuration.request.dto';
import { CommandBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const deleteConfigurationRequest: DeleteConfigurationRequestDto = {
  domain: Domain.AD,
  key: 'seatsProposed',
};

const mockCommandBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => {
      throw new DatabaseErrorException();
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Delete Configuration Grpc Controller', () => {
  let deleteConfigurationGrpcController: DeleteConfigurationGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        DeleteConfigurationGrpcController,
      ],
    }).compile();

    deleteConfigurationGrpcController =
      module.get<DeleteConfigurationGrpcController>(
        DeleteConfigurationGrpcController,
      );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(deleteConfigurationGrpcController).toBeDefined();
  });

  it('should delete a configuration item', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    await deleteConfigurationGrpcController.delete(deleteConfigurationRequest);
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if configuration item does not exist', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteConfigurationGrpcController.delete(
        deleteConfigurationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.NOT_FOUND);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if a database error occurs', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteConfigurationGrpcController.delete(
        deleteConfigurationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.INTERNAL);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteConfigurationGrpcController.delete(
        deleteConfigurationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });
});
